<?php
/**
 * Gestion du formulaire de chargement ou de vidage des tables de contours.
 *
 * @package    SPIP\CONTOURS\OBJET
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

if (!defined('_CONTOUR_MAX_CHOIX')) {
	define('_CONTOUR_MAX_CHOIX', 1);
}

/**
 * Chargement des données : le formulaire propose les actions possibles sur les tables de contours,
 * à savoir, charger ou vider et la liste des tables regroupées par service.
 * L'action vider s'appelle décharger car il existe dékà une fonction d'administration de vidage des tables.
 *
 * @uses services_geometrie_acquerir()
 * @uses territoire_est_peuple()
 * @uses contour_est_peuple()
 *
 * @return array Tableau des données à charger par le formulaire (affichage). Aucune donnée chargée n'est un
 *               champ de saisie, celle-ci sont systématiquement remises à zéro.
 *               - `_actions_contour`: (affichage) alias et libellés des actions possibles, `peupler` et `depeupler`
 *               - `_actions_disable`   : (affichage) liste des actions désactivées (`depeupler` si aucun pays n`est chargé)
 *               - `_action_defaut`     : (affichage) action sélectionnée par défaut, `peupler`
 *               - `_contours`          : (affichage) la liste des services de géométrie disponibles.
 *               - `_max_choix`         : (affichage) nombre maximum de choix.
 */
function formulaires_peupler_contours_charger() : array {
	// Initialisation des valeurs à transmettre au formulaire
	$valeurs = [];

	// Lister les actions sur les tables
	$valeurs['_actions_contour'] = [
		'peupler'   => _T('territoires:option_peupler_action'),
		'depeupler' => _T('territoires:option_depeupler_action')
	];
	$valeurs['_contours'] = [];

	// Acquérir les services disponibles pour les countours
	include_spip('inc/contours_utils');
	$services = services_geometrie_acquerir();

	// Ranger les services en fonction du type de territoire et du pays.
	include_spip('inc/territoire');
	include_spip('inc/contour');
	include_spip('action/editer_objet');
	$un_contour_charge = false;
	$options = ['champ_id' => 'iso_territoire', 'champs' => 'nom_usage'];
	foreach ($services as $_service => $_config) {
		if (territoire_est_peuple($_config['tags']['type'], $_config['tags']['pays'])) {
			if ($_config['tags']['type'] === 'subdivision') {
				$options[] = ['champ_id' => 'iso_territoire'];
				$groupe = _T('territoires:explication_subdivision_territoire')
					. " - {$_config['tags']['pays']} - "
					. extraire_multi(objet_lire('territoire', $_config['tags']['pays'], $options));
			} else {
				$groupe = _T('territoires:explication_zone_country_territoire');
			}

			if (!isset($valeurs['_contours'][$groupe])) {
				$valeurs['_contours'][$groupe] = [];
			}
			$cle = "{$_config['tags']['type']}:{$_config['tags']['pays']}:{$_service}";
			$valeurs['_contours'][$groupe][$cle] = extraire_multi($_config['title']);
			if (contour_est_peuple($_config['tags']['type'], $_config['tags']['pays'], $_service)) {
				$valeurs['_contours'][$groupe][$cle] .= ' - <em>[' . _T('territoires:info_territoire_peuple') . ']</em>';
				$un_contour_charge = true;
			}
		}
	}
	ksort($valeurs['_contours']);

	// Limiter les choix pour éviter un problème de timeout
	$valeurs['_max_choix'] = _CONTOUR_MAX_CHOIX;

	// Désactiver l'action vider si aucun pays encore chargé
	if (!$un_contour_charge) {
		$valeurs['_actions_disable'] = ['depeupler' => 'oui'];
		$valeurs['_action_defaut'] = 'peupler';
	}

	return $valeurs;
}

/**
 * Vérification des saisies : il est indispensable de choisir une action (`retirer` ou `ajouter`) et
 * un pays.
 *
 * @return array Tableau des erreurs sur l'action et/ou le pays ou tableau vide si aucune erreur.
 */
function formulaires_peupler_contours_verifier() : array {
	$erreurs = [];

	$obligatoires = ['contours', 'action_contour'];
	foreach ($obligatoires as $_obligatoire) {
		if (!_request($_obligatoire)) {
			$erreurs[$_obligatoire] = _T('info_obligatoire');
		}
	}

	return $erreurs;
}

/**
 * Exécution du formulaire : les pays choisis sont soit vidés, soit chargés.
 *
 * @uses contour_peupler()
 * @uses contour_depeupler()
 * @uses formulaires_peupler_contours_notifier()
  *
 * @return array Tableau retourné par le formulaire contenant toujours un message de bonne exécution ou
 *               d'erreur. L'indicateur editable est toujours à vrai.
 */
function formulaires_peupler_contours_traiter() : array {
	// Acquisition des saisies: comme elles sont obligatoires, il existe toujours une action et un pays.
	$action = _request('action_contour');
	$contours = _request('contours');

	// On boucle sur tous les ensemble de contours choisis.
	$statuts = [];
	foreach ($contours as $_contour) {
		// On identifie exatement le type de territoire et le pays si c'est une subdivision
		[$type, $pays, $service] = explode(':', $_contour);

		// Pour chaque pays, on génère l'action demandée.
		// (La fonction de chargement lance un vidage préalable si le pays demandé est déjà chargée)
		include_spip('inc/contour');
		$actionner = "contour_{$action}";
		$statuts[] = $actionner($type, $pays, $service);
	}

	// Formater le message correspondant au traitement du type
	$retour = formulaires_peupler_contours_notifier($action, $statuts);
	$retour['editable'] = true;

	return $retour;
}

/**
 * Formate les messages de succès et d'erreur résultant des actions de chargement ou de vidage
 * des contours.
 *
 * @param string $action  Action venant d'être appliquée à certains pays. Peut prendre les valeurs `peupler` et
 *                        `depeupler`.
 * @param array  $statuts Tableau résultant de l'action sur les tables choisies:
 *                        - `ok`  : `true` si le vidage a réussi, `false` sinon.
 *                        - `nok` : liste des pays en erreur ou tableau vide sinon.
 *                        - `sha` : liste des pays inchangés (SHA identique) ou tableau vide sinon.
 *                        Uniquement disponible pour l'action `peupler`.
 *
 * @return array Tableau des messages à afficher sur le formulaire:
 *               - `message_ok`     : message sur les services ayant été traités avec succès ou tableau vide sinon.
 *               - `message_erreur` : message sur les services en erreur ou tableau vide sinon.
 */
function formulaires_peupler_contours_notifier(string $action, array $statuts) : array {
	// Initialisation des messages de retour
	$messages = [
		'message_ok'     => '',
		'message_erreur' => ''
	];

	$variables = [
		'ok'  => [],
		'nok' => [],
		'sha' => [],
	];
	$statut_global = [
		'ok'  => false,
		'nok' => false,
		'sha' => false,
	];

	// Acquérir les services disponibles pour les countours
	include_spip('inc/contours_utils');
	$services = services_geometrie_acquerir();

	// On compile la liste des pays traités et un indicateur global pour chaque cas d'erreur.
	// Traitement des succès
	foreach ($statuts as $_statut) {
		// Extraction du service
		$service = $services[$_statut['service']]['title'];

		// Traitement des retours
		if (!empty($_statut['sha'])) {
			$variables['sha'][] = $service;
			$statut_global['sha'] = true;
		} elseif (!$_statut['ok']) {
			$variables['nok'][] = $service;
			$statut_global['nok'] = true;
		} else {
			$variables['ok'][] = $service;
			$statut_global['ok'] = true;
		}
	}

	// Traitement des succès
	if ($statut_global['ok']) {
		$messages['message_ok'] .= _T("contours:msg_{$action}_succes", ['service' => implode(', ', $variables['ok'])]);
	}

	// Traitement des erreurs
	if ($statut_global['nok']) {
		$messages['message_erreur'] .= _T("contours:msg_{$action}_erreur", ['service' => implode(', ', $variables['nok'])]);
	}
	if ($statut_global['sha']) {
		$messages['message_erreur'] .= $messages['message_erreur'] ? '<br />' : '';
		$messages['message_erreur'] .= _T("contours:msg_{$action}_notice", ['service' => implode(', ', $variables['sha'])]);
	}

	return $messages;
}
