<?php
/**
 * Fonctions utilitaires du plugin Contours appelées par les API.
 *
 * @package SPIP\CONTOURS\OUTILS
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

if (!defined('_CONTOUR_URL_BASE_ISOCODE')) {
	define('_CONTOUR_URL_BASE_ISOCODE', 'https://contrib.spip.net/http.api/ezrest');
}

/**
 * Acquiert les données de contours en provenance de l'API REST de Nomenclatures.
 * Contours utilise un cache du plugin Cache Factory afin de limiter les flux échangés avec le serveur Nomenclatures
 * sachant que les données sont volumineuses et changent peu.
 *
 * @param string $type    Type de territoires à peupler. Prends les valeurs `zone`, `country` ou `subdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays dont on veut peupler les subdivisions.
 *                        N'est utilisé que si le type choisi est `subdivision`.
 * @param string $service Identifiant du service Nomenclatures ayant permis le chargement. Est utilisé pour indexer
 *                        la consignation dans le cas des contours uniquement. Sinon vaut chaine vide
 *
 * @return array Tableau des contours pour les territoires demandés.
 */
function contours_acquerir(string $type, string $pays, string $service) : array {
	// Initialisation du tableau de sortie.
	$contours = [];

	// Lecture du cache si il existe et test de sa validité
	include_spip('inc/ezcache_cache');
	$cache = [
		'type'    => $type,
		'pays'    => $pays,
		'service' => $service
	];
	if (!$fichier_cache = cache_est_valide('contours', 'geodata', $cache)) {
		// Construire les filtres permettant ensuite de calculer l'url de la requête
		$filtres = [];
		if ($type) {
			$filtres['type'] = $type;
		}
		if ($pays) {
			$filtres['pays'] = $pays;
		}
		if ($service) {
			$filtres['service'] = $service;
		}

		// Ajouter les exclusions si nécessaire
		if (!empty($options['exclure'])) {
			$filtres['exclure'] = $options['exclure'];
		}

		// Appel à l'API REST de Nomenclatures
		$requeter = charger_fonction('requeter_isocode', 'inc');
		if ($data = $requeter(_CONTOUR_URL_BASE_ISOCODE, 'contours', $filtres)) {
			// On extrait uniquement l'index des données.
			$contours = $data['donnees'];

			// Mise à jour du cache correspondant
			cache_ecrire('contours', 'geodata', $cache, json_encode($contours));
		}
	} else {
		// Lecture des données du fichier cache valide
		$contours = cache_lire('contours', 'geodata', $fichier_cache);
	}

	return $contours;
}

/**
 * Liste les feeds de catégorie countours (map).
 *
 * @return array Liste des services.
 */
function services_geometrie_acquerir() : array {
	// Initialisation du tableau de sortie.
	$services = [];

	// Construire les filtres permettant ensuite de calculer l'url de la requête
	$filtres = [
		'categorie' => 'map',
	];

	// Appel à l'API REST de Nomenclatures
	$requeter = charger_fonction('requeter_isocode', 'inc');
	if ($data = $requeter(_CONTOUR_URL_BASE_ISOCODE, 'feeds', $filtres)) {
		$services = $data['donnees'];
		// On normalise les feeds en faisant en sorte que le tag pays soit toujours présent
		foreach ($services as $_service => $_config) {
			if (!isset($_config['tags']['pays'])) {
				$services[$_service]['tags']['pays'] = '';
			}
		}
	}

	return $services;
}

/**
 * Identifie la variable de configuration à partir du type de territoire, du pays et du service éventuellement.
 *
 * @param string $type    Type de territoires à peupler. Prends les valeurs `zone`, `country` ou `subdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays dont on veut peupler les subdivisions.
 *                        N'est utilisé que si le type choisi est `subdivision`.
 * @param string $service Identifiant du service Nomenclatures ayant permis le chargement. Est utilisé pour indexer
 *                        la consignation dans le cas des contours uniquement. Sinon vaut chaine vide
 *
 * @return string Identifiant de la variable de consigne.
 */
function consigne_contour_identifier($type, $pays, $service) {
	return ($type === 'subdivision')
		? "{$type}/{$pays}/{$service}"
		: "{$type}/{$service}";
}
